/*==============================================================================
 *
 *  The CHT_Value_Chart program is free software: you can redistribute
 *  it and/or modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  CHT_Value_Chart is distributed in the hope that it will be useful, but WITHOUT ANY
 *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 *  A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CHT_Value_Chart. If not, see <http://www.gnu.org/licenses/>.
 *
 * FILE: CHT_Value_Chart_v2.5.mq4
 * VERSION: 2.5
 * AUTHOR: Thierry Chappuis <tc77@pygnol.ch>
 * DESCRIPTION:
 * MT4 Custom indicator that aims at calculating and displaying a value chart
 *
 * LOG:
 * - 20121104: version 1.0
 * - 20121207: version 2.0, improvement of the display
 * - 20121211: version 2.1, multiple timeframes version
 * - 20121228: version 2.2, bug fix with vc_delete(). VC_Bars2Check has been added.
 * - 20130110: version 2.3, add support for automatic vc support/resistance display
 * - 20130110: version 2.3.1 bug fix in the value chart computation algorithm
 * - 20130110: version 2.4, bug fixes with multiple timeframes functionalities
 * - 20130122: version 2.5, integration of the alert system
 * - 20130313: version 3.0, integration of a new color scheme
 * - 20130512: version 4.0a1, optimisation of the performance
 * - 20130528: version 4.0a2, correction of the ZeroDivide error
 *
 * Copyright  2012-2013 Thierry Chappuis
 */

#property copyright "Copyright  2012-2013, Thierry Chappuis"
#property indicator_separate_window

#property indicator_buffers 7

#property indicator_color1 White

#property indicator_level1 6
#property indicator_level2 8
#property indicator_level3 -6
#property indicator_level4 -8
#property indicator_level5 10
#property indicator_level6 -10
#property indicator_level7 12
#property indicator_level8 -12

#property indicator_levelstyle 2
#property indicator_levelcolor DimGray

#property indicator_maximum 15
#property indicator_minimum -15

/* Input parameters */
extern int VC_Bars2Check = 1000;
extern bool VC_DisplayChart = true;
extern bool VC_DisplayZigZag = true;
extern bool VC_UseClassicColorSheme = true;
extern color VC_UpColor = Lime;
extern color VC_DownColor = Red;
extern int VC_WickWidth = 1;
extern int VC_BodyWidth = 4;

extern int ExtDepth = 5;
extern int ExtDeviation = 3;
extern int ExtBackstep = 1;

extern double VC_Overbought = 8;
extern double VC_SlightlyOverbought = 6;
extern double VC_Oversold = -8;
extern double VC_SlightlyOversold = -6;

extern color VC_OverboughtColor = Red;
extern color VC_SlightlyOverboughtColor = Orange;
extern color VC_OversoldColor = Lime;
extern color VC_SlightlyOversoldColor = Green;
extern color VC_NeutralColor = DimGray;

/* Global constants */

/* Buffers */
double ZigzagBuffer[];
double HighMapBuffer[];
double LowMapBuffer[];

double vcHigh[];
double vcLow[];
double vcOpen[];
double vcClose[];

/* Global variables */
string indicator_short_name;
int bar_new_ID;

// Value Chart ZigZag
int level = 3; // recounting's depth
bool downloadhistory = false;

double vcPoint = 0.1;

/* Initialisation of the indicator */
int init()
{
    indicator_short_name = "CHT_ValueChart";

    IndicatorShortName(indicator_short_name);

    if (VC_DisplayZigZag)
    {
        SetIndexStyle(0, DRAW_SECTION);
    }
    else
    {
        SetIndexStyle(0, DRAW_NONE);
    }
    SetIndexStyle(1, DRAW_NONE);
    SetIndexStyle(2, DRAW_NONE);
    SetIndexStyle(3, DRAW_NONE);
    SetIndexStyle(4, DRAW_NONE);
    SetIndexStyle(5, DRAW_NONE);
    SetIndexStyle(6, DRAW_NONE);
    SetIndexStyle(7, DRAW_NONE);

    SetIndexBuffer(0, ZigzagBuffer);
    SetIndexBuffer(1, HighMapBuffer);
    SetIndexBuffer(2, LowMapBuffer);
    SetIndexBuffer(3, vcHigh);
    SetIndexBuffer(4, vcLow);
    SetIndexBuffer(5, vcOpen);
    SetIndexBuffer(6, vcClose);

    SetIndexEmptyValue(0, 0.0);
    SetIndexEmptyValue(1, 0.0);
    SetIndexEmptyValue(2, 0.0);
    SetIndexEmptyValue(3, 0.0);
    SetIndexEmptyValue(4, 0.0);
    SetIndexEmptyValue(5, 0.0);

    string name;
    for (int i = ObjectsTotal() - 1; i >= 0; i--)
    {
        name = ObjectName(i);
        string s = "CHT_VC_";

        if (StringSubstr(name, 0, StringLen(s)) == s)
        {
            ObjectDelete(name);
        }
    }

    return (0);
}

/* Entry point for the indicator action */
int start()
{
    vc();

    if (VC_DisplayChart== true)
    {
        if (VC_UseClassicColorSheme)
        {
            displays_vc();
        }
        else
        {
            displays_vc2();
        }
    }

    if (VC_DisplayZigZag)
    {
        vcZigZag();
    }

    return(0);
}

int deinit()
{
    vc_delete();
    Comment("");
    return (0);
}

int vcHighest(int period, int start=0)
{
    int max = start;

    for (int i = start; i < start+period; i++)
    {
        if (vcHigh[i] > vcHigh[max])
        {
            max = i;
        }
    }

    return (max);
}

int vcLowest(int period, int start=0)
{
    int min = start;

    for (int i = start; i < start+period; i++)
    {
        if (vcLow[i] < vcLow[min])
        {
            min = i;
        }
    }

    return (min);
}

int vc()
{
    int counted_bars = IndicatorCounted();
    int bars = Bars - 5;

    double floatingAxis;
    double volatilityUnit;

    if (counted_bars > 0)
    {
        counted_bars--;
        bars = Bars - counted_bars;
    }


    for (int i = bars; i >= 0; i--)
    {
        double sumHigh = High[i] + High[i+1] + High[i+2] + High[i+3] + High[i+4];
        double sumLow = Low[i] + Low[i+1] + Low[i+2] + Low[i+3] + Low[i+4];

        // Determination of the floating axis
        floatingAxis = 0.1 * (sumHigh + sumLow);

        /* Determination of the volatility unit */
        volatilityUnit = 0.04 * (sumHigh - sumLow);

        // Determination of relative high, low, open and close values
        vcHigh[i] = 0;
        vcLow[i] = 0;
        vcOpen[i] = 0;
        vcClose[i] = 0;

        if (volatilityUnit != 0)
        {
            vcHigh[i] = (High[i] - floatingAxis) / volatilityUnit;
            vcLow[i] = (Low[i] - floatingAxis) / volatilityUnit;
            vcOpen[i] = (Open[i] - floatingAxis) / volatilityUnit;
            vcClose[i] = (Close[i] - floatingAxis) / volatilityUnit;
        }
    }

    return (0);
}

int vcZigZag()
{
    int i;
    int counted_bars = IndicatorCounted();
    int limit;
    int counterZ;
    int whatlookfor;
    int shift;
    int back;
    int lasthighpos, lastlowpos;
    double val, res;
    double curlow, curhigh, lasthigh, lastlow;

    if (counted_bars == 0 && downloadhistory)
    {
        ArrayInitialize(ZigzagBuffer, 0.0);
        ArrayInitialize(HighMapBuffer, 0.0);
        ArrayInitialize(LowMapBuffer, 0.0);
    }
    if (counted_bars == 0)
    {
        limit = Bars-ExtDepth;
        downloadhistory = true;
    }
    if (counted_bars > 0)
    {
        while (counterZ < level && i < 100)
        {
            res = ZigzagBuffer[i];
            if (res!=0)
            {
                counterZ++;
            }
            i++;
        }
        i--;
        limit = i;

        if (LowMapBuffer[i] != 0)
        {
            curlow = LowMapBuffer[i];
            whatlookfor = 1;
        }
        else
        {
            curhigh = HighMapBuffer[i];
            whatlookfor = -1;
        }

        for (i = limit-1; i >= 0; i--)
        {
            ZigzagBuffer[i] = 0.0;
            LowMapBuffer[i] = 0.0;
            HighMapBuffer[i] = 0.0;
        }
    }

    for(shift = limit; shift >= 0; shift--)
    {
        val = vcLow[vcLowest(ExtDepth,shift)];
        if (val == lastlow)
        {
            val = 0.0;
        }
        else
        {
            lastlow = val;
            if ((vcLow[shift]-val) > (ExtDeviation*vcPoint))
            {
                val = 0.0;
            }
            else
            {
                for (back = 1; back <= ExtBackstep; back++)
                {
                    res = LowMapBuffer[shift+back];
                    if ((res!=0) && (res>val))
                    {
                        LowMapBuffer[shift+back] = 0.0;
                    }
                }
            }
        }
        if (vcLow[shift] == val)
        {
            LowMapBuffer[shift] = val;
        }
        else
        {
            LowMapBuffer[shift] = 0.0;
        }

        //--- high
        val = vcHigh[vcHighest(ExtDepth,shift)];
        if (val == lasthigh)
        {
            val = 0.0;
        }
        else
        {
            lasthigh = val;
            if ((val-vcHigh[shift]) > (ExtDeviation*vcPoint))
            {
                val = 0.0;
            }
            else
            {
                for (back = 1; back <= ExtBackstep; back++)
                {
                    res = HighMapBuffer[shift+back];
                    if ((res!=0)&&(res<val))
                    {
                        HighMapBuffer[shift+back] = 0.0;
                    }
                }
            }
        }
        if (vcHigh[shift] == val)
        {
            HighMapBuffer[shift] = val;
        }
        else
        {
            HighMapBuffer[shift] = 0.0;
        }
    }

    // final cutting
    if (whatlookfor == 0)
    {
        lastlow = 0;
        lasthigh = 0;
    }
    else
    {
        lastlow = curlow;
        lasthigh = curhigh;
    }
    for (shift = limit; shift >= 0; shift--)
    {
        res = 0.0;
        switch(whatlookfor)
        {
        case 0: // look for peak or lawn
            if (lastlow == 0 && lasthigh == 0)
            {
                if (HighMapBuffer[shift] != 0)
                {
                    lasthigh = vcHigh[shift];
                    lasthighpos = shift;
                    whatlookfor = -1;
                    ZigzagBuffer[shift] = lasthigh;
                    res = 1;
                }
                if (LowMapBuffer[shift] != 0)
                {
                    lastlow = vcLow[shift];
                    lastlowpos = shift;
                    whatlookfor = 1;
                    ZigzagBuffer[shift] = lastlow;
                    res = 1;
                }
            }
            break;
        case 1: // look for peak
            if (LowMapBuffer[shift] != 0.0 && LowMapBuffer[shift] < lastlow && HighMapBuffer[shift] == 0.0)
            {
                ZigzagBuffer[lastlowpos] = 0.0;
                lastlowpos = shift;
                lastlow = LowMapBuffer[shift];
                ZigzagBuffer[shift] = lastlow;
                res = 1;
            }
            if (HighMapBuffer[shift] != 0.0 && LowMapBuffer[shift] == 0.0)
            {
                lasthigh = HighMapBuffer[shift];
                lasthighpos = shift;
                ZigzagBuffer[shift] = lasthigh;
                whatlookfor = -1;
                res = 1;
            }
            break;
        case -1: // look for lawn
            if (HighMapBuffer[shift] != 0.0 && HighMapBuffer[shift] > lasthigh && LowMapBuffer[shift] == 0.0)
            {
                ZigzagBuffer[lasthighpos] = 0.0;
                lasthighpos = shift;
                lasthigh = HighMapBuffer[shift];
                ZigzagBuffer[shift] = lasthigh;
            }
            if (LowMapBuffer[shift] != 0.0 && HighMapBuffer[shift] == 0.0)
            {
                lastlow = LowMapBuffer[shift];
                lastlowpos = shift;
                ZigzagBuffer[shift] = lastlow;
                whatlookfor = 1;
            }
            break;
        default:
            return;
        }
    }

    return(0);
}

void displays_vc()
{
    int window = WindowFind(indicator_short_name);

    string current_body_ID;
    string current_wick_ID;

    int counted_bars = IndicatorCounted();
    int bars = Bars - 5;

    if (counted_bars > 0)
    {
        counted_bars--;
        bars = Bars - counted_bars;
    }

    if (VC_Bars2Check > 0)
    {
        bars = MathMin(bars, VC_Bars2Check);
    }

    for(int i = 0; i < bars; i++)
    {

        current_body_ID = "CHT_VC_(" + 5 + "," + 0 + ")" + "_BODY_ID(" + Time[i] + ")";
        current_wick_ID = "CHT_VC_(" + 5 + "," + 0 + ")" + "_WICK_ID(" + Time[i] + ")";

        ObjectDelete(current_body_ID);
        ObjectDelete(current_wick_ID);

        ObjectCreate(current_body_ID, OBJ_TREND, window, Time[i], vcOpen[i], Time[i], vcClose[i]);
        ObjectSet(current_body_ID, OBJPROP_STYLE, STYLE_SOLID);
        ObjectSet(current_body_ID, OBJPROP_RAY, false);
        ObjectSet(current_body_ID, OBJPROP_WIDTH, VC_BodyWidth);


        ObjectCreate(current_wick_ID, OBJ_TREND, window, Time[i], vcHigh[i], Time[i], vcLow[i]);
        ObjectSet(current_wick_ID, OBJPROP_STYLE, STYLE_SOLID);
        ObjectSet(current_wick_ID, OBJPROP_RAY, false);
        ObjectSet(current_wick_ID, OBJPROP_WIDTH, VC_WickWidth);

        if (vcOpen[i] <= vcClose[i])
        {
            ObjectSet(current_body_ID, OBJPROP_COLOR, VC_UpColor);
            ObjectSet(current_wick_ID, OBJPROP_COLOR, VC_UpColor);
        }
        else
        {
            ObjectSet(current_body_ID, OBJPROP_COLOR, VC_DownColor);
            ObjectSet(current_wick_ID, OBJPROP_COLOR, VC_DownColor);
        }
    }
}

void displays_vc2()
{
    int window = WindowFind(indicator_short_name);

    string current_body_ID;
    string current_wick_ID;
    
    int counted_bars = IndicatorCounted();
    int bars = Bars - 5;

    if (counted_bars > 0)
    {
        counted_bars--;
        bars = Bars - counted_bars;
    }

    if (VC_Bars2Check > 0)
    {
        bars = MathMin(bars, VC_Bars2Check);
    }

    for(int i = 0; i < bars; i++)
    {

        if (vcHigh[i] == 0.0 && vcOpen[i] == 0.0 && vcClose[i] == 0.0 && vcLow[i] == 0.0)
        {
            // Do nothing
        }
        else
        {
            current_body_ID = "CHT_VC_(" + 5 + "," + 0 + ")" + "_BODY_ID(" + Time[i] + ")_";
            current_wick_ID = "CHT_VC_(" + 5 + "," + 0 + ")" + "_WICK_ID(" + Time[i] + ")_";

            vc_delete_current_candle(i);

            double structure[5][2];
            structure[0][0] = VC_Overbought;
            structure[0][1] = 20;
            structure[1][0] = VC_SlightlyOverbought;
            structure[1][1] = VC_Overbought;
            structure[2][0] = VC_SlightlyOversold;
            structure[2][1] = VC_SlightlyOverbought;
            structure[3][0] = VC_Oversold;
            structure[3][1] = VC_SlightlyOversold;
            structure[4][0] = -20;
            structure[4][1] = VC_Oversold;

            color colors[5];
            colors[0] = VC_OverboughtColor;
            colors[1] = VC_SlightlyOverboughtColor;
            colors[2] = VC_NeutralColor;
            colors[3] = VC_SlightlyOversoldColor;
            colors[4] = VC_OversoldColor;

            double body[5][2] = {100, 100, 100, 100, 100, 100, 100, 100, 100, 100};
            double wick[5][2] = {100, 100, 100, 100, 100, 100, 100, 100, 100, 100};

            double low = vcLow[i];
            double high = vcHigh[i];
            double blow = MathMin(vcClose[i], vcOpen[i]);
            double bhigh = MathMax(vcClose[i], vcOpen[i]);

            for (int m = 0; m < 5; m++)
            {
                int slow = structure[m][0];
                int shigh = structure[m][1];

                // Body low
                if (blow < slow && bhigh > slow)
                {
                    body[m][0] = structure[m][0];
                }
                else if (blow >= slow && blow < shigh)
                {
                    body[m][0] = blow;
                }
                else
                {
                    // Do nothing
                }

                // Body high
                if (bhigh > shigh && blow < shigh)
                {
                    body[m][1] = structure[m][1];
                }
                else if (bhigh > slow && bhigh <= shigh)
                {
                    body[m][1] = bhigh;
                }
                else
                {
                    // Do nothing
                }

                // Wick low
                if (low < slow && high > slow)
                {
                    wick[m][0] = structure[m][0];
                }
                else if (low >= slow && low < shigh)
                {
                    wick[m][0] = low;
                }
                else
                {
                    // Do nothing
                }

                // Wick high
                if (high > shigh && low < shigh)
                {
                    wick[m][1] = structure[m][1];
                }
                else if (high > slow && high <= shigh)
                {
                    wick[m][1] = high;
                }
                else
                {
                    // Do nothing
                }
            }


            for (m = 0; m < 5; m++)
            {
                if (wick[m][0] < 100 && wick[m][1] < 100)
                {
                    draw_wick(current_wick_ID + m, i, wick[m][0], wick[m][1], colors[m]);
                }

                if (body[m][0] < 100 && body[m][1] < 100)
                {
                    draw_body(current_body_ID + m, i, body[m][0], body[m][1], colors[m]);
                }
            }
        }
    }
}

void draw_body(string id, int i, double open, double close, color col)
{
    int window = WindowFind(indicator_short_name);

    ObjectCreate(id, OBJ_TREND, window, Time[i], open, Time[i], close);
    ObjectSet(id, OBJPROP_STYLE, STYLE_SOLID);
    ObjectSet(id, OBJPROP_RAY, false);
    ObjectSet(id, OBJPROP_WIDTH, VC_BodyWidth);
    ObjectSet(id, OBJPROP_COLOR, col);
}

void draw_wick(string id, int i, double open, double close, color col)
{
    int window = WindowFind(indicator_short_name);

    ObjectCreate(id, OBJ_TREND, window, Time[i], open, Time[i], close);
    ObjectSet(id, OBJPROP_STYLE, STYLE_SOLID);
    ObjectSet(id, OBJPROP_RAY, false);
    ObjectSet(id, OBJPROP_WIDTH, VC_WickWidth);
    ObjectSet(id, OBJPROP_COLOR, col);
}

void vc_delete_current_candle(int shift)
{
    string name;
    for (int i = ObjectsTotal() - 1; i >= 0; i--)
    {
        name = ObjectName(i);
        string s = "CHT_VC_(" + 5 + "," + 0 + ")" + "_BODY_ID(" + Time[shift] + ")_";

        if (StringSubstr(name, 0, StringLen(s)) == s)
        {
            ObjectDelete(name);
        }
        
        s = "CHT_VC_(" + 5 + "," + 0 + ")" + "_WICK_ID(" + Time[shift] + ")_";

        if (StringSubstr(name, 0, StringLen(s)) == s)
        {
            ObjectDelete(name);
        }
    }
}

void vc_delete()
{
    string name;
    for (int i = ObjectsTotal() - 1; i >= 0; i--)
    {
        name = ObjectName(i);
        string s = "CHT_VC_(" + 5 + "," + 0 + ")";

        if (StringSubstr(name, 0, StringLen(s)) == s)
        {
            ObjectDelete(name);
        }
    }
}


